@@ -211,10 +211,12 @@ gboolean              g_setenv             (const gchar *variable,
 					    gboolean     overwrite);
 void                  g_unsetenv           (const gchar *variable);
 gchar**               g_listenv            (void);
+
+/* private */
 const gchar*	     _g_getenv_nomalloc	   (const gchar	*variable,
 					    gchar        buffer[1024]);
 
-/* we try to provide a usefull equivalent for ATEXIT if it is
+/* we try to provide a useful equivalent for ATEXIT if it is
  * not defined, but use is actually abandoned. people should
  * use g_atexit() instead.
  */
@@ -303,7 +305,8 @@ G_INLINE_FUNC guint
 g_bit_storage (gulong number)
 {
 #if defined(__GNUC__) && (__GNUC__ >= 4) && defined(__OPTIMIZE__)
-  return number ? GLIB_SIZEOF_LONG * 8 - __builtin_clzl(number) : 1;
+  return G_LIKELY (number) ?
+	   ((GLIB_SIZEOF_LONG * 8 - 1) ^ __builtin_clzl(number)) + 1 : 1;
 #else
   register guint n_bits = 0;
   
@@ -409,25 +412,14 @@ DllMain (HINSTANCE hinstDLL,						\
 	 LPVOID    lpvReserved)						\
 {									\
   wchar_t wcbfr[1000];							\
-  char cpbfr[1000];							\
   char *tem;								\
   switch (fdwReason)							\
     {									\
     case DLL_PROCESS_ATTACH:						\
-      if (GetVersion () < 0x80000000)					\
-	{								\
-	  GetModuleFileNameW ((HMODULE) hinstDLL, wcbfr, G_N_ELEMENTS (wcbfr));	\
-	  tem = g_utf16_to_utf8 (wcbfr, -1, NULL, NULL, NULL);		\
-	  dll_name = g_path_get_basename (tem);				\
-	  g_free (tem);							\
-	}								\
-      else								\
-	{								\
-	  GetModuleFileNameA ((HMODULE) hinstDLL, cpbfr, G_N_ELEMENTS (cpbfr));	\
-	  tem = g_locale_to_utf8 (cpbfr, -1, NULL, NULL, NULL);		\
-	  dll_name = g_path_get_basename (tem);				\
-	  g_free (tem);							\
-	}								\
+      GetModuleFileNameW ((HMODULE) hinstDLL, wcbfr, G_N_ELEMENTS (wcbfr)); \
+      tem = g_utf16_to_utf8 (wcbfr, -1, NULL, NULL, NULL);		\
+      dll_name = g_path_get_basename (tem);				\
+      g_free (tem);							\
       break;								\
     }									\
 									\
