@@ -82,19 +82,20 @@ gchar*          g_win32_error_message (gint error);
 #define g_win32_get_package_installation_directory g_win32_get_package_installation_directory_utf8
 #define g_win32_get_package_installation_subdirectory g_win32_get_package_installation_subdirectory_utf8
 
-gchar*          g_win32_get_package_installation_directory (gchar *package,
-							    gchar *dll_name);
+gchar*          g_win32_get_package_installation_directory (const gchar *package,
+							    const gchar *dll_name);
 
-gchar*          g_win32_get_package_installation_subdirectory (gchar *package,
-							       gchar *dll_name,
-							       gchar *subdir);
+gchar*          g_win32_get_package_installation_subdirectory (const gchar *package,
+							       const gchar *dll_name,
+							       const gchar *subdir);
 
 guint		g_win32_get_windows_version (void);
 
 gchar*          g_win32_locale_filename_from_utf8 (const gchar *utf8filename);
 
-#define G_WIN32_IS_NT_BASED() (g_win32_get_windows_version () < 0x80000000)
-#define G_WIN32_HAVE_WIDECHAR_API() (G_WIN32_IS_NT_BASED ())
+/* As of GLib 2.14 we only support NT-based Windows */
+#define G_WIN32_IS_NT_BASED() TRUE
+#define G_WIN32_HAVE_WIDECHAR_API() TRUE
 
 G_END_DECLS
 
